@@ -182,21 +182,34 @@ class OpenStackConfig(object):
             self.cloud_config = dict(
                 clouds=dict(defaults=dict(self.defaults)))
 
-        self._cache_max_age = 0
+        self._cache_expiration_time = 0
         self._cache_path = CACHE_PATH
         self._cache_class = 'dogpile.cache.null'
         self._cache_arguments = {}
+        self._cache_expiration = {}
         if 'cache' in self.cloud_config:
-            self._cache_max_age = self.cloud_config['cache'].get(
-                'max_age', self._cache_max_age)
-            if self._cache_max_age:
+            cache_settings = self._normalize_keys(self.cloud_config['cache'])
+
+            # expiration_time used to be 'max_age' but the dogpile setting
+            # is expiration_time. Support max_age for backwards compat.
+            self._cache_expiration_time = cache_settings.get(
+                'expiration_time', cache_settings.get(
+                    'max_age', self._cache_expiration_time))
+
+            # If cache class is given, use that. If not, but if cache time
+            # is given, default to memory. Otherwise, default to nothing.
+            # to memory.
+            if self._cache_expiration_time:
                 self._cache_class = 'dogpile.cache.memory'
-            self._cache_path = os.path.expanduser(
-                self.cloud_config['cache'].get('path', self._cache_path))
             self._cache_class = self.cloud_config['cache'].get(
                 'class', self._cache_class)
-            self._cache_arguments = self.cloud_config['cache'].get(
+
+            self._cache_path = os.path.expanduser(
+                cache_settings.get('path', self._cache_path))
+            self._cache_arguments = cache_settings.get(
                 'arguments', self._cache_arguments)
+            self._cache_expiration = cache_settings.get(
+                'expiration', self._cache_expiration)
 
     def _load_config_file(self):
         return self._load_yaml_file(self._config_files)
@@ -221,8 +234,14 @@ class OpenStackConfig(object):
                 new_config[key] = value
         return new_config
 
+    def get_cache_expiration_time(self):
+        return self._cache_expiration_time
+
+    def get_cache_interval(self):
+        return self._cache_expiration_time
+
     def get_cache_max_age(self):
-        return self._cache_max_age
+        return self._cache_expiration_time
 
     def get_cache_path(self):
         return self._cache_path
@@ -231,7 +250,10 @@ class OpenStackConfig(object):
         return self._cache_class
 
     def get_cache_arguments(self):
-        return self._cache_arguments
+        return self._cache_arguments.copy()
+
+    def get_cache_expiration(self):
+        return self._cache_expiration.copy()
 
     def _get_regions(self, cloud):
         if cloud not in self.cloud_config['clouds']:
@@ -433,6 +455,14 @@ class OpenStackConfig(object):
                 if d_opt_name in config:
                     return config[d_opt_name]
 
+    def auth_config_hook(self, config):
+        """Allow examination of config values before loading auth plugin
+
+        OpenStackClient will override this to perform additional chacks
+        on auth_type.
+        """
+        return config
+
     def _get_auth_loader(self, config):
         # Re-use the admin_token plugin for the "None" plugin
         # since it does not look up endpoints or tokens but rather
@@ -540,6 +570,7 @@ class OpenStackConfig(object):
             An argparse Namespace object; allows direct passing in of
             argparse options to be added to the cloud config.  Values
             of None and '' will be removed.
+        :param region_name: Name of the region of the cloud.
         :param kwargs: Additional configuration options
 
         :raises: keystoneauth1.exceptions.MissingRequiredOptions
@@ -561,7 +592,15 @@ class OpenStackConfig(object):
 
         # Regions is a list that we can use to create a list of cloud/region
         # objects. It does not belong in the single-cloud dict
-        config.pop('regions', None)
+        regions = config.pop('regions', None)
+        if regions and args['region_name'] not in regions:
+            raise exceptions.OpenStackConfigException(
+                'Region {region_name} is not a valid region name for cloud'
+                ' {cloud}. Valid choices are {region_list}. Please note that'
+                ' region names are case sensitive.'.format(
+                    region_name=args['region_name'],
+                    region_list=','.join(regions),
+                    cloud=cloud))
 
         # Can't just do update, because None values take over
         for (key, val) in iter(args.items()):
@@ -576,6 +615,11 @@ class OpenStackConfig(object):
                 if type(config[key]) is not bool:
                     config[key] = get_boolean(config[key])
 
+        # NOTE(dtroyer): OSC needs a hook into the auth args before the
+        #                plugin is loaded in order to maintain backward-
+        #                compatible behaviour
+        config = self.auth_config_hook(config)
+
         if loading:
             if validate:
                 try:
@@ -616,7 +660,9 @@ class OpenStackConfig(object):
             name=cloud_name, region=config['region_name'],
             config=self._normalize_keys(config),
             force_ipv4=force_ipv4,
-            auth_plugin=auth_plugin)
+            auth_plugin=auth_plugin,
+            openstack_config=self
+            )
 
     @staticmethod
     def set_one_cloud(config_file, cloud, set_config=None):
